/*
 *      ts - This program outputs a file/record description of a tape's
 *      structure.
 *
 *      Exits:  0 - normal.
 *              1 - usage error.
 *              2 - tape read error.
 *              3 - SIGINT received.
 *
 *      Copyright (C) 1983, 1984 Lyle McElhaney
 *      Permission to copy for non-commercial use granted under condition
 *      that this notice remains intact in the copy.
 *
 *      Address: 2489 W. Ridge Rd., Littleton, CO 80120
 *      ....denelcor!lmc
 */
/****************************************************************
 * ts (device)  -- device = device to use (default = /dev/tape)	*
 ****************************************************************/
#include <sys/types.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/mtio.h>
#include <signal.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#define MAXBUF (62 * 1024)

/*      This can be whatever maximum the machine/os will take.
 *      16000 is allowed on an 11/44 running System 3 Unix (tm).
 */

#ifndef YES
#define YES 1
#define NO 0
#endif

char           *tape = "/dev/tape";
char            buff[MAXBUF];
int             tapedev;
long            recno, nrec, fsize, filen;

static void sig(int dummy)
{
    printf("    Records %ld through %ld (%ld records), length = %ld bytes\n",
		  nrec, recno - 1, recno - nrec, fsize);
    printf("ts: aborted by interrupt at that point.\n");
    close(tapedev);
    exit(3);
}

int main(int argc, char *argv[])
{
    int             n;
    int             maxbuf = MAXBUF;

    if ((argc > 2) && (strcmp(argv[1], "-c") == 0)) {
	maxbuf = (10240);
	argc--;
	argv++;
    }
    switch (argc) {
	case 1:
	    break;
	case 2:
	    tape = argv[1];
	    break;
	default:
	    fprintf(stderr, "Usage: ts [device]\n");
	    exit(2);
    }

    signal(SIGINT, sig);
    tapedev = open(tape, 0);
    if (tapedev < 0) {
	fprintf(stderr, "ts: cannot open %s\n", tape);
	exit(2);
    }

    filen = 0;
    for (;;) {
	recno = nrec = 0;
	for (;;) {
	    n = read(tapedev, buff, maxbuf);
	    if (n > 0) {
		if (recno == 0) {
		    printf("File %ld:\n", filen);
		    fsize = n;
		}
		if (n != fsize) {
		    printf("    Records %ld through %ld (%ld records), length = %ld bytes\n",
			nrec, recno - 1, recno - nrec, fsize);
		    fsize = n;
		    nrec = recno;
		}
		recno++;
	    } else
		goto Out;
	}
Out:	if (n == 0) {
	    if (recno != 0)
		printf("    Records %ld through %ld (%ld records), length = %ld bytes\n",
			nrec, recno - 1, recno - nrec, fsize);
	} else {
	    printf("Tape read error %d in record %ld\n", n, recno);
	    exit(1);
	}
	if (recno == 0)
	    break;
	filen++;
    }
    printf("End of tape reached.\n");
    close(tapedev);
    return(0);
}

